/****************************************************************************
** Meta object code from reading C++ file 'kfindtreeview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kfind-17.04.3/src/kfindtreeview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kfindtreeview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KFindSortFilterProxyModel_t {
    QByteArrayData data[1];
    char stringdata0[26];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KFindSortFilterProxyModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KFindSortFilterProxyModel_t qt_meta_stringdata_KFindSortFilterProxyModel = {
    {
QT_MOC_LITERAL(0, 0, 25) // "KFindSortFilterProxyModel"

    },
    "KFindSortFilterProxyModel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KFindSortFilterProxyModel[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void KFindSortFilterProxyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    Q_UNUSED(_o);
    Q_UNUSED(_id);
    Q_UNUSED(_c);
    Q_UNUSED(_a);
}

const QMetaObject KFindSortFilterProxyModel::staticMetaObject = {
    { &QSortFilterProxyModel::staticMetaObject, qt_meta_stringdata_KFindSortFilterProxyModel.data,
      qt_meta_data_KFindSortFilterProxyModel,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KFindSortFilterProxyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KFindSortFilterProxyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KFindSortFilterProxyModel.stringdata0))
        return static_cast<void*>(const_cast< KFindSortFilterProxyModel*>(this));
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int KFindSortFilterProxyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    return _id;
}
struct qt_meta_stringdata_KFindTreeView_t {
    QByteArrayData data[15];
    char stringdata0[227];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KFindTreeView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KFindTreeView_t qt_meta_stringdata_KFindTreeView = {
    {
QT_MOC_LITERAL(0, 0, 13), // "KFindTreeView"
QT_MOC_LITERAL(1, 14, 14), // "resultSelected"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 13), // "copySelection"
QT_MOC_LITERAL(4, 44, 20), // "contextMenuRequested"
QT_MOC_LITERAL(5, 65, 1), // "p"
QT_MOC_LITERAL(6, 67, 19), // "deleteSelectedFiles"
QT_MOC_LITERAL(7, 87, 24), // "moveToTrashSelectedFiles"
QT_MOC_LITERAL(8, 112, 11), // "slotExecute"
QT_MOC_LITERAL(9, 124, 5), // "index"
QT_MOC_LITERAL(10, 130, 19), // "slotExecuteSelected"
QT_MOC_LITERAL(11, 150, 20), // "openContainingFolder"
QT_MOC_LITERAL(12, 171, 11), // "saveResults"
QT_MOC_LITERAL(13, 183, 24), // "reconfigureMouseSettings"
QT_MOC_LITERAL(14, 208, 18) // "updateMouseButtons"

    },
    "KFindTreeView\0resultSelected\0\0"
    "copySelection\0contextMenuRequested\0p\0"
    "deleteSelectedFiles\0moveToTrashSelectedFiles\0"
    "slotExecute\0index\0slotExecuteSelected\0"
    "openContainingFolder\0saveResults\0"
    "reconfigureMouseSettings\0updateMouseButtons"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KFindTreeView[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   72,    2, 0x0a /* Public */,
       4,    1,   73,    2, 0x0a /* Public */,
       6,    0,   76,    2, 0x08 /* Private */,
       7,    0,   77,    2, 0x08 /* Private */,
       8,    1,   78,    2, 0x08 /* Private */,
      10,    0,   81,    2, 0x08 /* Private */,
      11,    0,   82,    2, 0x08 /* Private */,
      12,    0,   83,    2, 0x08 /* Private */,
      13,    0,   84,    2, 0x08 /* Private */,
      14,    0,   85,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QModelIndex,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void KFindTreeView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KFindTreeView *_t = static_cast<KFindTreeView *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->resultSelected((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->copySelection(); break;
        case 2: _t->contextMenuRequested((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 3: _t->deleteSelectedFiles(); break;
        case 4: _t->moveToTrashSelectedFiles(); break;
        case 5: _t->slotExecute((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 6: _t->slotExecuteSelected(); break;
        case 7: _t->openContainingFolder(); break;
        case 8: _t->saveResults(); break;
        case 9: _t->reconfigureMouseSettings(); break;
        case 10: _t->updateMouseButtons(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KFindTreeView::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KFindTreeView::resultSelected)) {
                *result = 0;
                return;
            }
        }
    }
}

const QMetaObject KFindTreeView::staticMetaObject = {
    { &QTreeView::staticMetaObject, qt_meta_stringdata_KFindTreeView.data,
      qt_meta_data_KFindTreeView,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KFindTreeView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KFindTreeView::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KFindTreeView.stringdata0))
        return static_cast<void*>(const_cast< KFindTreeView*>(this));
    return QTreeView::qt_metacast(_clname);
}

int KFindTreeView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void KFindTreeView::resultSelected(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
