/****************************************************************************
** Meta object code from reading C++ file 'kfinddlg.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kfind-17.04.3/src/kfinddlg.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kfinddlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KfindDlg_t {
    QByteArrayData data[15];
    char stringdata0[181];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KfindDlg_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KfindDlg_t qt_meta_stringdata_KfindDlg = {
    {
QT_MOC_LITERAL(0, 0, 8), // "KfindDlg"
QT_MOC_LITERAL(1, 9, 11), // "haveResults"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 14), // "resultSelected"
QT_MOC_LITERAL(4, 37, 11), // "startSearch"
QT_MOC_LITERAL(5, 49, 10), // "stopSearch"
QT_MOC_LITERAL(6, 60, 9), // "newSearch"
QT_MOC_LITERAL(7, 70, 8), // "addFiles"
QT_MOC_LITERAL(8, 79, 32), // "QList<QPair<KFileItem,QString> >"
QT_MOC_LITERAL(9, 112, 8), // "setFocus"
QT_MOC_LITERAL(10, 121, 10), // "slotResult"
QT_MOC_LITERAL(11, 132, 5), // "about"
QT_MOC_LITERAL(12, 138, 14), // "slotDeleteItem"
QT_MOC_LITERAL(13, 153, 12), // "slotNewItems"
QT_MOC_LITERAL(14, 166, 14) // "finishAndClose"

    },
    "KfindDlg\0haveResults\0\0resultSelected\0"
    "startSearch\0stopSearch\0newSearch\0"
    "addFiles\0QList<QPair<KFileItem,QString> >\0"
    "setFocus\0slotResult\0about\0slotDeleteItem\0"
    "slotNewItems\0finishAndClose"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KfindDlg[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       3,    1,   77,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   80,    2, 0x0a /* Public */,
       5,    0,   81,    2, 0x0a /* Public */,
       6,    0,   82,    2, 0x0a /* Public */,
       7,    1,   83,    2, 0x0a /* Public */,
       9,    0,   86,    2, 0x0a /* Public */,
      10,    1,   87,    2, 0x0a /* Public */,
      11,    0,   90,    2, 0x0a /* Public */,
      12,    1,   91,    2, 0x0a /* Public */,
      13,    1,   94,    2, 0x0a /* Public */,
      14,    0,   97,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,

       0        // eod
};

void KfindDlg::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KfindDlg *_t = static_cast<KfindDlg *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->haveResults((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->resultSelected((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->startSearch(); break;
        case 3: _t->stopSearch(); break;
        case 4: _t->newSearch(); break;
        case 5: _t->addFiles((*reinterpret_cast< const QList<QPair<KFileItem,QString> >(*)>(_a[1]))); break;
        case 6: _t->setFocus(); break;
        case 7: _t->slotResult((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->about(); break;
        case 9: _t->slotDeleteItem((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->slotNewItems((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->finishAndClose(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KfindDlg::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KfindDlg::haveResults)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (KfindDlg::*_t)(bool );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KfindDlg::resultSelected)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject KfindDlg::staticMetaObject = {
    { &KDialog::staticMetaObject, qt_meta_stringdata_KfindDlg.data,
      qt_meta_data_KfindDlg,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KfindDlg::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KfindDlg::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KfindDlg.stringdata0))
        return static_cast<void*>(const_cast< KfindDlg*>(this));
    return KDialog::qt_metacast(_clname);
}

int KfindDlg::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void KfindDlg::haveResults(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KfindDlg::resultSelected(bool _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_END_MOC_NAMESPACE
