/****************************************************************************
** Meta object code from reading C++ file 'kftabdlg.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kfind-17.04.3/src/kftabdlg.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kftabdlg.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KfindTabWidget_t {
    QByteArrayData data[13];
    char stringdata0[187];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KfindTabWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KfindTabWidget_t qt_meta_stringdata_KfindTabWidget = {
    {
QT_MOC_LITERAL(0, 0, 14), // "KfindTabWidget"
QT_MOC_LITERAL(1, 15, 11), // "startSearch"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 8), // "setFocus"
QT_MOC_LITERAL(4, 37, 29), // "slotUpdateDateLabelsForNumber"
QT_MOC_LITERAL(5, 67, 5), // "value"
QT_MOC_LITERAL(6, 73, 27), // "slotUpdateDateLabelsForType"
QT_MOC_LITERAL(7, 101, 5), // "index"
QT_MOC_LITERAL(8, 107, 22), // "slotUpdateByteComboBox"
QT_MOC_LITERAL(9, 130, 12), // "getDirectory"
QT_MOC_LITERAL(10, 143, 9), // "fixLayout"
QT_MOC_LITERAL(11, 153, 18), // "slotSizeBoxChanged"
QT_MOC_LITERAL(12, 172, 14) // "slotEditRegExp"

    },
    "KfindTabWidget\0startSearch\0\0setFocus\0"
    "slotUpdateDateLabelsForNumber\0value\0"
    "slotUpdateDateLabelsForType\0index\0"
    "slotUpdateByteComboBox\0getDirectory\0"
    "fixLayout\0slotSizeBoxChanged\0"
    "slotEditRegExp"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KfindTabWidget[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   60,    2, 0x0a /* Public */,
       4,    1,   61,    2, 0x0a /* Public */,
       6,    1,   64,    2, 0x0a /* Public */,
       8,    1,   67,    2, 0x0a /* Public */,
       9,    0,   70,    2, 0x08 /* Private */,
      10,    0,   71,    2, 0x08 /* Private */,
      11,    1,   72,    2, 0x08 /* Private */,
      12,    0,   75,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

       0        // eod
};

void KfindTabWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KfindTabWidget *_t = static_cast<KfindTabWidget *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->startSearch(); break;
        case 1: _t->setFocus(); break;
        case 2: _t->slotUpdateDateLabelsForNumber((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->slotUpdateDateLabelsForType((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->slotUpdateByteComboBox((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->getDirectory(); break;
        case 6: _t->fixLayout(); break;
        case 7: _t->slotSizeBoxChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->slotEditRegExp(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KfindTabWidget::*_t)();
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KfindTabWidget::startSearch)) {
                *result = 0;
                return;
            }
        }
    }
}

const QMetaObject KfindTabWidget::staticMetaObject = {
    { &KTabWidget::staticMetaObject, qt_meta_stringdata_KfindTabWidget.data,
      qt_meta_data_KfindTabWidget,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KfindTabWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KfindTabWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KfindTabWidget.stringdata0))
        return static_cast<void*>(const_cast< KfindTabWidget*>(this));
    return KTabWidget::qt_metacast(_clname);
}

int KfindTabWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KTabWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KfindTabWidget::startSearch()
{
    QMetaObject::activate(this, &staticMetaObject, 0, Q_NULLPTR);
}
struct qt_meta_stringdata_KDigitValidator_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KDigitValidator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KDigitValidator_t qt_meta_stringdata_KDigitValidator = {
    {
QT_MOC_LITERAL(0, 0, 15) // "KDigitValidator"

    },
    "KDigitValidator"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KDigitValidator[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void KDigitValidator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    Q_UNUSED(_o);
    Q_UNUSED(_id);
    Q_UNUSED(_c);
    Q_UNUSED(_a);
}

const QMetaObject KDigitValidator::staticMetaObject = {
    { &QValidator::staticMetaObject, qt_meta_stringdata_KDigitValidator.data,
      qt_meta_data_KDigitValidator,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KDigitValidator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KDigitValidator::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KDigitValidator.stringdata0))
        return static_cast<void*>(const_cast< KDigitValidator*>(this));
    return QValidator::qt_metacast(_clname);
}

int KDigitValidator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QValidator::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    return _id;
}
QT_END_MOC_NAMESPACE
