/****************************************************************************
** Meta object code from reading C++ file 'kquery.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.6.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kfind-17.04.3/src/kquery.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kquery.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.6.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
struct qt_meta_stringdata_KQuery_t {
    QByteArrayData data[15];
    char stringdata0[233];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_KQuery_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_KQuery_t qt_meta_stringdata_KQuery = {
    {
QT_MOC_LITERAL(0, 0, 6), // "KQuery"
QT_MOC_LITERAL(1, 7, 13), // "foundFileList"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 32), // "QList<QPair<KFileItem,QString> >"
QT_MOC_LITERAL(4, 55, 6), // "result"
QT_MOC_LITERAL(5, 62, 15), // "slotListEntries"
QT_MOC_LITERAL(6, 78, 9), // "KIO::Job*"
QT_MOC_LITERAL(7, 88, 17), // "KIO::UDSEntryList"
QT_MOC_LITERAL(8, 106, 10), // "slotResult"
QT_MOC_LITERAL(9, 117, 5), // "KJob*"
QT_MOC_LITERAL(10, 123, 12), // "slotCanceled"
QT_MOC_LITERAL(11, 136, 27), // "slotreadyReadStandardOutput"
QT_MOC_LITERAL(12, 164, 26), // "slotreadyReadStandardError"
QT_MOC_LITERAL(13, 191, 20), // "slotendProcessLocate"
QT_MOC_LITERAL(14, 212, 20) // "QProcess::ExitStatus"

    },
    "KQuery\0foundFileList\0\0"
    "QList<QPair<KFileItem,QString> >\0"
    "result\0slotListEntries\0KIO::Job*\0"
    "KIO::UDSEntryList\0slotResult\0KJob*\0"
    "slotCanceled\0slotreadyReadStandardOutput\0"
    "slotreadyReadStandardError\0"
    "slotendProcessLocate\0QProcess::ExitStatus"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_KQuery[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       4,    1,   62,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    1,   65,    2, 0x0a /* Public */,
       5,    2,   68,    2, 0x09 /* Protected */,
       8,    1,   73,    2, 0x09 /* Protected */,
      10,    1,   76,    2, 0x09 /* Protected */,
      11,    0,   79,    2, 0x09 /* Protected */,
      12,    0,   80,    2, 0x09 /* Protected */,
      13,    2,   81,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::QStringList,    2,
    QMetaType::Void, 0x80000000 | 6, 0x80000000 | 7,    2,    2,
    QMetaType::Void, 0x80000000 | 9,    2,
    QMetaType::Void, 0x80000000 | 9,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 14,    2,    2,

       0        // eod
};

void KQuery::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        KQuery *_t = static_cast<KQuery *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->foundFileList((*reinterpret_cast< QList<QPair<KFileItem,QString> >(*)>(_a[1]))); break;
        case 1: _t->result((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->slotListEntries((*reinterpret_cast< QStringList(*)>(_a[1]))); break;
        case 3: _t->slotListEntries((*reinterpret_cast< KIO::Job*(*)>(_a[1])),(*reinterpret_cast< const KIO::UDSEntryList(*)>(_a[2]))); break;
        case 4: _t->slotResult((*reinterpret_cast< KJob*(*)>(_a[1]))); break;
        case 5: _t->slotCanceled((*reinterpret_cast< KJob*(*)>(_a[1]))); break;
        case 6: _t->slotreadyReadStandardOutput(); break;
        case 7: _t->slotreadyReadStandardError(); break;
        case 8: _t->slotendProcessLocate((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KIO::Job* >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KJob* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< KJob* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        void **func = reinterpret_cast<void **>(_a[1]);
        {
            typedef void (KQuery::*_t)(QList<QPair<KFileItem,QString> > );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KQuery::foundFileList)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (KQuery::*_t)(int );
            if (*reinterpret_cast<_t *>(func) == static_cast<_t>(&KQuery::result)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject KQuery::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_KQuery.data,
      qt_meta_data_KQuery,  qt_static_metacall, Q_NULLPTR, Q_NULLPTR}
};


const QMetaObject *KQuery::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KQuery::qt_metacast(const char *_clname)
{
    if (!_clname) return Q_NULLPTR;
    if (!strcmp(_clname, qt_meta_stringdata_KQuery.stringdata0))
        return static_cast<void*>(const_cast< KQuery*>(this));
    return QObject::qt_metacast(_clname);
}

int KQuery::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void KQuery::foundFileList(QList<QPair<KFileItem,QString> > _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void KQuery::result(int _t1)
{
    void *_a[] = { Q_NULLPTR, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_END_MOC_NAMESPACE
